function fxDTSBrick::createTriggerZone(%brick,%triggerData)
{
   if(!isObject(%triggerData))
   {
      error("ERROR: Trigger data \"" @ %triggerData @ "\" does not exist for " @ %brick.dataBlock.getName() @ " brick!");
      return;
   }
   if(%triggerData.getClassName() !$= "TriggerData")
   {
      error("ERROR: \"" @ %triggerData @ "\" is not a TriggerData datablock!");
      return;
   }
   // new Trigger(t){datablock=PLFinishTriggerData;polyhedron=$p;};
   %t = new Trigger()
   {
      dataBlock = %triggerData;
      polyhedron = "0 0 0 1 0 0 0 -1 0 0 0 1";
   };
   if(!isObject(%t))
   {
      error("ERROR: Could not create trigger for " @ %brick.dataBlock.getName() @ " brick!");
      return;
   }
   missionCleanup.add(%t);
		
   //The trigger is made slightly larger than the brick and offset
   %boxMin = getWords(%brick.getWorldBox(), 0, 2);
   %boxMax = getWords(%brick.getWorldBox(), 3, 5);
   %boxDiff = vectorSub(%boxMax, %boxMin);
   %boxDiff = vectorAdd(%boxDiff, "0 0 0.2"); 
   %t.setScale(%boxDiff);
   
   %posA = %brick.getWorldBoxCenter();
   %posB = %t.getWorldBoxCenter();
   %posDiff = vectorSub(%posA, %posB);
   %posDiff = vectorAdd(%posDiff, "0 0 0.1");
   %t.setTransform(%posDiff);
   
   %brick.triggerZone = %t;
   %t.spawnBrick = %brick;
}

datablock fxDTSBrickData(brickPLStartData : brickVehicleSpawnData)
{
	category = "Special";
	subCategory = "Block Bastion";
	uiName = "Payload Start";
	isTDMBrick = 1;
};
datablock fxDTSBrickData(brickPLTrackData : brickVehicleSpawnData)
{
	category = "Special";
	subCategory = "Block Bastion";
	uiName = "Payload Track";
	isTDMBrick = 1;
	specialbricktype = "";
	bricktype = 0;
};
datablock fxDTSBrickData(brickPLFinishData : brickVehicleSpawnData)
{
	category = "Special";
	subCategory = "Block Bastion";
	uiName = "Payload Goal";
	isTDMBrick = 1;
	bricktype = 0;
	specialbricktype = "";
};
datablock TSShapeConstructor(PLBombCartDTS)
{
	baseShape  = "./Bombcart.dts";
	sequence0  = "./B_root.dsq root";

	sequence1  = "./B_walk.dsq run";
	sequence2  = "./B_walk.dsq walk";
	sequence3  = "./B_root.dsq back";
	sequence4  = "./B_root.dsq side";

	sequence5  = "./B_root.dsq crouch";
	sequence6  = "./B_root.dsq crouchRun";
	sequence7  = "./B_root.dsq crouchBack";
	sequence8  = "./B_root.dsq crouchSide";

	sequence9  = "./B_root.dsq look";
	sequence10 = "./B_root.dsq headside";
	sequence11 = "./B_root.dsq headUp";

	sequence12 = "./B_root.dsq jump";
	sequence13 = "./B_root.dsq standjump";
	sequence14 = "./B_root.dsq fall";
	sequence15 = "./B_root.dsq land";

	sequence16 = "./B_root.dsq armAttack";
	sequence17 = "./B_root.dsq armReadyLeft";
	sequence18 = "./B_root.dsq armReadyRight";
	sequence19 = "./B_root.dsq armReadyBoth";
	sequence20 = "./B_root.dsq spearready";  
	sequence21 = "./B_root.dsq spearThrow";

	sequence22 = "./B_root.dsq talk";  

	sequence23 = "./B_root.dsq death1"; 
	
	sequence24 = "./B_root.dsq shiftUp";
	sequence25 = "./B_root.dsq shiftDown";
	sequence26 = "./B_root.dsq shiftAway";
	sequence27 = "./B_root.dsq shiftTo";
	sequence28 = "./B_root.dsq shiftLeft";
	sequence29 = "./B_root.dsq shiftRight";
	sequence30 = "./B_root.dsq rotCW";
	sequence31 = "./B_root.dsq rotCCW";

	sequence32 = "./B_root.dsq undo";
	sequence33 = "./B_root.dsq plant";

	sequence34 = "./B_root.dsq sit";

	sequence35 = "./B_root.dsq wrench";

	sequence36 = "./B_root.dsq activate";
	sequence37 = "./B_root.dsq activate2";

	sequence38 = "./B_root.dsq leftrecoil";
};
datablock PlayerData(PLBombcartArmor : PlayerStandardArmor)
{
	shapeFile = "./Bombcart.dts";
	aiAvoidThis = true;
	mass = 500;
	maxDamage = 100000;
	maxEnergy =  10;
	repairRate = 100;
	runForce = 28 * 90 * 10;
	maxForwardSpeed = 3 * 10;
	maxBackwardSpeed = 1.5 * 10;
	maxSideSpeed = 0 * 10;
	maxForwardCrouchSpeed = 0;
	maxBackwardCrouchSpeed = 0;
	maxSideCrouchSpeed = 0;
	maxForwardProneSpeed = 0;
	maxBackwardProneSpeed = 0;
	maxSideProneSpeed = 0;
	maxForwardWalkSpeed = 0;
	maxBackwardWalkSpeed = 0;
	maxSideWalkSpeed = 0;
	maxUnderwaterForwardSpeed = 0;
	maxUnderwaterBackwardSpeed = 0;
	maxUnderwaterSideSpeed = 0;
	jumpForce = 0; //8.3 * 90;
	jumpEnergyDrain = 0;
	minJumpEnergy = 0;
	jumpDelay = 0;
	minJetEnergy = 0;
	jetEnergyDrain = 0;
	canJet = 0;
	speedDamageScale = 0;
	uiName = "Bombcart";
	rideable = true;
	paintable = true;
	numMountPoints = 0;
};
function PLBombcartArmor::damage(%a,%b,%c,%d,%e,%f,%g,%h)
{
	return;
}
function AIPlayer::findBombTrack(%bomb,%startBrick)
{
	%currentLook = %bomb.getAimLocation();
	%startPos = %startBrick.getTransform();
	switch(%startBrick.angleID)
	{
		case 0:
			%endPos = strReplace(%startPos,getWord(%startPos,1),getWord(%startPos,1) + 4);
		case 1:
			%endPos = strReplace(%startPos,getWord(%startPos,0),getWord(%startPos,0) + 4);
		case 2:
			%endPos = strReplace(%startPos,getWord(%startPos,1),getWord(%startPos,1) - 4);
		case 3:
			%endPos = strReplace(%startPos,getWord(%startPos,0),getWord(%startPos,0) - 4);
	}
	%mask = ($TypeMasks::FxBrickAlwaysObjectType);
	%ray = ContainerRayCast(%startPos, %endPos, %mask, %startBrick);
	%nextBrick = getWord(%ray,0);
	if(isObject(%nextBrick))
	{
		%bomb.setAimLocation(%nextBrick.getPosition());
	}
	%bomb.previousBrick = %startbrick;
	%bomb.nextBrick = %nextbrick;
}
function AIPlayer::setMoveUserCSpeed(%bomb,%c)
{
   %speed = 0;
   for(%i=0;%i<getFieldCount(%c);%i++)
   {
      if(getField(%c,%i).bb_class $= BB_CLASS_SCOUT)
         %speed = %speed + 0.15;
      else
         %speed = %speed + 0.1;
   }
   if(%speed > 1)
      %speed = 1;
   %bomb.setMoveY(%speed);
}
function AIPlayer::searchForPlayers(%bomb)
{
	cancel(%bomb.SFPschedule);
	%bomb.users = "";
	%bomb.setToMove = 0;
	if(!%bomb.reversing)
	   %bomb.setMoveY(0);
	%position = %bomb.getPosition();
	%boxSize = "1000 1000 1000";
	%mask = ($TypeMasks::PlayerObjectType);
	initContainerBoxSearch(%position,%boxSize,%mask);
	while(%searchObj = containerSearchNext())
	{
		if(%searchObj != %bomb && %searchObj.client.bb_team $= BB_TEAM_RED && isObject(%bomb.nextBrick))
	 	{
	 	   %dist = vectorDist(%bomb.getPosition(),%searchObj.getPosition());
	 	   if(strLen($PayloadPushDist) < 1)
	 	      $PayloadPushDist = 3;
         if(%dist > $PayloadPushDist)
            continue;
	 	   %bomb.users = trim(%bomb.users TAB %searchObj.client);
         %bomb.reversing = 0;
         %bomb.setMoveUserCSpeed(%bomb.users);
         %bomb.lastContactTime = $Sim::Time;
		}
	}
	if($Sim::Time - %bomb.lastContactTime >= 30 && !%bomb.reversing && %bomb.spawnBrick !$= %bomb.previousBrick)
	{
	   %bomb.reversing = 1;
	   %bomb.setMoveY(-0.05);
	   commandToAll('BB_PL_BombBackwards');
	}
	%bomb.SFPschedule = %bomb.schedule(500,"searchForPlayers");
}
datablock TriggerData(PLFinishTriggerData)
{
	tickPeriodMS = 100;
};
function PLFinishTriggerData::onEnterTrigger(%this,%trigger,%user)
{
	%client = %trigger.spawnBrick.getGroup().client;
	if(%user.getDataBlock() != nameToID("PLBombcartArmor") || !isEventPending(%user.SFPschedule))
	{
		return;
	}
	GameMode_PL_updateBombSettings(%user.spawnBrick,0);
	%str = "";
	for(%i=0;%i<getFieldCount(%user.users);%i++)
	{
	   %str = %str @ "\c0" @ getField(%user.users,%i).name @ "\c6, ";
	   clientAchProgress(getField(%user.users,%i),"Boom Box",getField(%user.users,%i).achievementProgress["Boom Box"]+1,3);
	}
	%str = getSubStr(%str,0,strLen(%str)-2);
	messageAll('',"\c0"@%str@" \c6"@(getFieldCount(%bomb.users) > 1 ? "have" : "has")@" set up the bomb.");
	if(strLen($PayloadExplosionSize) < 1)
	   $PayloadExplosionSize = 2;
	%user.spawnExplosion(rocketLauncherProjectile,$PayloadExplosionSize,getField(%user.users,getRandom(0,getFieldCount(%user.users)-1)));
	%user.schedule(500,delete);
	winRoundForTeam(BB_TEAM_RED,"set up payload before time ran out");
}
datablock TriggerData(PLContainerSearchTriggerData)
{
	tickPeriodMS = 100;
};
function PLContainerSearchTriggerData::onEnterTrigger(%this,%trigger,%user)
{
	if(%user.getDataBlock() != nameToID("PLBombcartArmor") || %trigger.spawnBrick $= "" || %user.modeStarted == false)
	{
		return;
	}
	%user.findBombTrack(%trigger.spawnBrick);
	%user.searchForPlayers();
}
function startGamemodeRound_PL(%time)
{
   for(%i=0;%i<MainBrickGroup.getCount();%i++)
   {
      %group = MainBrickGroup.getObject(%i);
      if(!isObject(%group))
				continue;
      for(%j=0;%j<%group.getCount();%j++)
      {
         %o = %group.getObject(%j);
         if(%o.getDataBlock().getName() $= "brickPLStartData")
         {
            %o.respawnVehicle();
            schedule(1000,0,"GameMode_PL_updateBombSettings",%o,1);
         }
      }
   }
}
function endGamemodeRound_PL()
{
		for(%i=0;%i<ClientGroup.getCount();%i++)
		{
			%group = ClientGroup.getObject(%i).brickGroup;
			if(!isObject(%group))
				continue;
			for(%j=0;%j<%group.getCount();%j++)
			{
				%o = %group.getObject(%j);
	 				if(%o.getDataBlock().getName() $= "brickPLStartData")
	 				{
	  					%o.respawnVehicle();
						schedule(1000,0,"GameMode_PL_updateBombSettings",%o,0);
	 				}
				}
  		}
}
function GameMode_PL_updateBombSettings(%spawn,%status)
{
	if(!isObject(%spawn.vehicle))
	{
		return;
	}
	if(%status == 1)
	{
		%spawn.vehicle.minigame = $BB::Minigame;
		%spawn.vehicle.searchForPlayers();
		%spawn.vehicle.findBombTrack(%spawn);
		%spawn.vehicle.modeStarted = true;
	}
	else
	{
		%spawn.vehicle.minigame = $BB::Minigame;
		cancel(%spawn.vehicle.SFPschedule);
		%spawn.vehicle.setMoveY(0);
		%spawn.vehicle.modeStarted = false;
	}
}
package GameMode_Payload
{
	function fxDTSBrick::onPlant(%this)
	{
		Parent::onPlant(%this);
		if(%this.getDataBlock() == nameToID("brickPLFinishData"))
		{
			%this.createTriggerZone(PLFinishTriggerData);
		}
		else if(%this.getDataBlock() == nameToID("brickPLTrackData"))
		{
			%this.createTriggerZone(PLContainerSearchTriggerData);
		}
		else if(%this.getDataBlock() == nameToID("brickPLStartData"))
		{
			%this.setVehicle(nameToID("PLBombcartArmor"));
		}
	}
	function fxDTSBrick::onRemove(%this)
	{
		if(isObject(%this.triggerZone))
		{
			%this.triggerZone.delete();
		}
		Parent::onRemove(%this);
	}
};
activatePackage(Gamemode_Payload);